import os.path
import unittest
import sqlparse
import re

from migration.connector.destination.clickzetta.destination import ClickZettaDestination


class TestUtil(unittest.TestCase):
    def test_sqlparse(self):
        sql = '''
        CREATE TABLE table2
(
    event_day DATE,
    siteid INT DEFAULT '10',
    citycode SMALLINT,
    username VARCHAR(32) DEFAULT '',
    pv BIGINT SUM DEFAULT '0'
)
UNIQUE    KEY (event_day, siteid, citycode, username)
PARTITION BY RANGE(`event_day`)
(
    PARTITION p201706 VALUES LESS THAN ('2017-07-01'),
    PARTITION p201707 VALUES LESS THAN ('2017-08-01'),
    PARTITION p201708 VALUES LESS THAN ('2017-09-01')
)
DISTRIBUTED BY HASH(`siteid`) BUCKETS 10
PROPERTIES("replication_num" = "1");
        '''
        sql = sqlparse.format(sql, reindent=True, keyword_case='upper')
        # result = re.match(r'(.*)AGGREGATE(.*)KEY(.*?)\((.*?)\).*', sql, re.S)
        result = re.match(r'(.*)DISTRIBUTED(.*)BY(.*?)\((.*?)\)(.*)BUCKETS(.*?)(\d+).*', sql, re.S)
        # result = re.match(r'(.*?)PARTITION(.*?)BY(.*?)\((.*?)\).*', sql, re.S)
        # result = re.match(r'(.*?)UNIQUE(.*?)KEY(.*?)\((.*?)\).*', sql, re.S)
        for column in result.group(4).strip().split(','):
            print(column.replace('`', '').strip())

        print(result.group(7).strip())
        print('vachar(32)'.upper())

    def test_list_spilt(self):
        class Box:
            def __init__(self, name):
                self.name = name

        boxes_tasks_map = {}
        tasks = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
        b = [tasks[i:i + 3] for i in range(0, len(tasks), 3)]

        print(b)

    def test_path(self):

        all_tasks = []
        a = ['a1', 'a2', 'a3']
        b = ['b1', 'b2', 'b3']
        c = ['c1', 'c2', 'c3']
        for i in range(3):
            all_tasks.extend((a[i], b[i], c[i]))
        print(all_tasks)

    def test_destination(self):
        PK_TABLE_DML_HINT = {'hints': {'cz.sql.allow.insert.table.with.pk': 'true'}}
        config = {'service': 'poc-ap-shanghai-tencentcloud.api.clickzetta.com', 'username': 'changean',
                  'workspace': 'quickStart_WS',
                  'password': 'Achangean123!', 'instance': '1fdbc4a7', 'vcluster': 'DEFAULT', "instanceId": 32}
        destination = ClickZettaDestination(config)
        print(destination.execute_sql('show schemas;'))
        print(os.path.join(os.path.abspath(os.path.dirname(os.path.dirname(__file__))), 'scripts/linux/meta_cmd'))
        list_1 = (8, 8, 8.0)
        list_2 = (8.0, 8.0, 8.0)
        print(list_1 == list_2)



import logging
from datetime import datetime

from migration.base.status import TaskType, TaskStatus
from migration.connector.destination.base import Destination
from migration.connector.source import Source
from migration.connector.source.enum import Column
from migration.scheduler.task import Task
from migration.util import migration_tasks_status

logger = logging.getLogger(__name__)
NUMBER_TYPE = ['BIGINT', 'DECIMAL', 'DOUBLE', 'FLOAT', 'INT', 'SMALLINT', 'TINYINT']
LEFT_BRACKET = '('


class ValidationTask(Task):
    def __init__(self, name: str, task_type: TaskType, source: Source, destination: Destination, *args, **kwargs):
        super().__init__(name, task_type, *args, **kwargs)
        self.source = source
        self.destination = destination

    def run(self):
        try:
            self.status = TaskStatus.RUNNING
            if not self.check_count():
                return self
            if not self.check_number_type_statistics():
                return self
            self.status = TaskStatus.COMPLETED
        except BaseException as e:
            self.status = TaskStatus.FAILED
            return self
        self.end_time = datetime.now()
        migration_tasks_status.update_task_status(self.destination, self)
        logger.info(f"ValidationTask {self.name} finished running, status: {self.status.value}")
        return self

    def check_count(self):
        sql = f"select count(*) from {self.name}"
        source_count = self.source.execute_sql(sql)[0]
        logger.info(f"source table: {self.name} count: {source_count}")
        destination_count = self.destination.execute_sql(sql)[0]
        logger.info(f"destination table {self.name} count: {destination_count}")
        if source_count != destination_count:
            self.status = TaskStatus.FAILED
            self.end_time = datetime.now()
            migration_tasks_status.update_task_status(self.destination, self)
            logger.error(
                f"ValidationTask {self.name} failed to run, error: count not equal, retry times: {self.retry_times}")
            return False
        return True

    def check_number_type_statistics(self):
        type_mapping = self.source.type_mapping()
        table_columns = self.source.get_table_columns(self.name.split('.')[0], self.name.split('.')[1])
        for column in table_columns:
            if self.is_number_type(column, type_mapping):
                sql = f"select min({column.name}), max({column.name}), avg({column.name}) from {self.name}"
                source_result = self.source.execute_sql(sql)[0]
                logger.info(f"source table: {self.name} column: {column.name} statistics: {source_result}")
                destination_result = self.destination.execute_sql(sql)[0]
                logger.info(f"destination table: {self.name} column: {column.name} statistics: {destination_result}")
                if source_result != destination_result:
                    self.status = TaskStatus.FAILED
                    self.end_time = datetime.now()
                    migration_tasks_status.update_task_status(self.destination, self)
                    logger.error(
                        f'ValidationTask {self.name} failed to run, error: number type statistics '
                        f'not equal, retry times: {self.retry_times}')
                    return False
        return True

    def check_string_type_statistics(self):
        pass

    def check_top_n(self):
        pass

    def is_number_type(self, column: Column, type_mapping: dict) -> bool:
        if LEFT_BRACKET in column.type:
            column_type = column.type.split(LEFT_BRACKET)[0]
            return type_mapping.get(column_type, column_type) in NUMBER_TYPE

        return type_mapping.get(column.type, column.type) in NUMBER_TYPE

    def default_profile_sql(self, type_mapping):
        columns = self.source.get_table_columns(self.name.split('.')[0], self.name.split('.')[1])
        profile_sql = f"with source_data data as (select * from {self.name}), \n" \
                      f"column_profiles  as ( \n"
        for index, column in enumerate(columns):
            profile_sql += f"select '{column.name}' as column_name, \n" \
                           f"'{column.type}' as column_type, \n" \
                           f"count(*) as row_count, \n" \
                           f"sum(case when {column.name} is null then 0 else 1 end) / count(*) as not_null_proportion,\n" \
                           f"count(distinct {column.name}) / count(*) as distinct_proportion, \n" \
                           f"count(distinct {column.name}) as distinct_count, \n" \
                           f"count(distinct {column.name}) = count(*) as is_unique, \n"
            if self.is_number_type(column, type_mapping):
                profile_sql += f"min({column.name}) as min_value, \n" \
                               f"max({column.name}) as max_value, \n" \
                               f"avg({column.name}) as avg_value \n" \
                               f"stddev_pop({column.name}) as stddev_pop_value \n" \
                               f"stddev_sample({column.name}) as stddev_sample_value \n"
            else:
                profile_sql += f"null as min_value, \n" \
                               f"null as max_value, \n" \
                               f"null as avg_value \n" \
                               f"null as stddev_pop_value \n" \
                               f"null as stddev_sample_value \n"
            profile_sql += f"from source_data \n"
            if index != len(columns) - 1:
                profile_sql += f"union all \n"
        profile_sql += f") \n" \
                       f"select * from column_profiles;"

        return profile_sql



import logging
from migration.base.exceptions import ProfileConfigError, GrammarRestrictionsError, SchedulerRuntimeError
from migration.scheduler.task.schema_task import SchemaMigrationTask
from migration.scheduler.task.base_task import TaskType
from migration.scheduler.transformer import Transformer
from migration.scheduler.scheduler import SchedulerConfig, Scheduler
import migration.util.migration_tasks_status as migration_tasks_status

logger = logging.getLogger(__name__)

DOT_SPLITTER = '.'
LEFT_BRACKET = '('


class SchemaTransformer(Transformer):
    def get_migration_tasks(self):
        logger.info("Start to get schema migration tasks")
        transform_table_list = self.get_transform_table_list()
        if not transform_table_list:
            logger.warning("No table to transform, please check your profile.yml")
            raise ProfileConfigError("No table to transform, please check your profile.yml")
        schema_migration_tables = []

        if DOT_SPLITTER not in transform_table_list[0]:
            for db in transform_table_list:
                tables = self.source.get_table_names(db)
                for table in tables:
                    schema_migration_tables.append(f"{db}.{table[0]}")
        else:
            for table in transform_table_list:
                if '*' in table:
                    db = table.split(DOT_SPLITTER)[0]
                    tables = self.source.get_table_names(db)
                    for result in tables:
                        schema_migration_tables.append(f"{db}.{result[0]}")
                else:
                    schema_migration_tables.append(table)
        logger.debug(f"schema migration tasks: {schema_migration_tables}")
        logger.info(f"schema migration tasks count: {len(schema_migration_tables)}")
        schema_migration_tasks = []
        for table in schema_migration_tables:
            schema_migration_tasks.append(self.construct_schema_task(table))

        return schema_migration_tasks

    def construct_schema_task(self, table_name):
        db, table = table_name.split(DOT_SPLITTER)
        type_mapping = self.source.type_mapping()
        for k, v in type_mapping.items():
            type_mapping.update({k.upper(): v.upper()})
        primary_keys = self.source.get_primary_key(db, table)
        cluster_info = self.source.get_table_cluster_info(db, table)
        partition_columns = self.source.get_table_partition_columns(db, table)
        columns = self.source.get_table_columns(db, table)
        for column in columns:
            if LEFT_BRACKET in column.type:
                column_type = column.type.split(LEFT_BRACKET)[0]
                if column_type in type_mapping:
                    column.type.replace(column_type, type_mapping.get(column_type))
            else:
                if column.type in type_mapping:
                    column.type = type_mapping.get(column.type)

        destination_ddl = self.destination.gen_destination_ddl(db, table, columns, primary_keys, cluster_info,
                                                               partition_columns)
        task = SchemaMigrationTask(name=table_name, ddl=destination_ddl, task_type=TaskType.SCHEMA_MIGRATION,
                                   project_id=self.project_id, destination=self.destination)
        migration_tasks_status.init_task_status(self.destination, task)
        return task

    def schedule_migration_tasks(self):
        logger.info("Start to schedule schema migration tasks")
        migration_tasks = self.get_migration_tasks()
        if migration_tasks:
            if len(migration_tasks) < self.transform_concurrency:
                schedule_config = SchedulerConfig(quit_if_failed=self.quit_if_fail)
                scheduler = Scheduler(schedule_config)
                self.schedules_tasks_map[scheduler] = migration_tasks
            else:
                for i in range(self.transform_concurrency):
                    schedule_config = SchedulerConfig(quit_if_failed=self.quit_if_fail)
                    scheduler = Scheduler(schedule_config)
                    self.schedules_tasks_map[scheduler] = migration_tasks[i::self.transform_concurrency]



import logging
import re
import time
from datetime import datetime

import pymysql
import sqlparse

from migration.connector.source.base import Source
from migration.base.exceptions import SourceExecutionError
from migration.connector.source.enum import Column, ClusterInfo
import migration.util.object_storage_util as object_storage_util

logger = logging.getLogger(__name__)


class DorisSource(Source):
    def __init__(self, config: dict, meta_conf_path=None, storage_conf_path=None):
        super().__init__('Doris', config)
        self.connection_params = self.get_connection_params()
        self.meta_conf_path = meta_conf_path
        self.storage_config_path = storage_conf_path

    """
    Doris connection parameters is a dict including following keys:
    1. host: host of Doris
    2. port: port for mysql client, default is 9030
    3. database: database name
    4. user: user name
    5. passwd: password
    """

    def get_connection_params(self):
        assert self.config['fe_servers']
        assert self.config['user']
        assert self.config['password']

        return {
            'host': self.config['fe_servers'][0].split(':')[0],
            'port': int(self.config['fe_servers'][0].split(':')[1]),
            'user': self.config['user'],
            'password': self.config['password']
        }

    def connect(self):
        if self.connection is None:
            self.connection = pymysql.connect(**self.connection_params)
            logger.info(f"Connect to Doris {self.connection_params['host']} successfully")

    def get_database_names(self):
        result = self.execute_sql("show databases")
        return [row[0] for row in result]

    def get_table_names(self, database_name):
        result = self.execute_sql(f"show tables from {database_name}")
        return [row[0] for row in result]

    def get_ddl_sql(self, database_name, table_name):
        return self.execute_sql(f"show create table {database_name}.{table_name}")[0][1]

    def get_table_cluster_info(self, database_name, table_name):
        ddl_sql = self.get_ddl_sql(database_name, table_name)
        cluster_columns = []
        ddl_format_sql = sqlparse.format(ddl_sql, reindent=True, keyword_case='upper')
        match_result = re.match(r'(.*)DISTRIBUTED(.*)BY(.*?)\((.*?)\)(.*)BUCKETS(.*?)(\d+).*', ddl_format_sql, re.S)
        if match_result:
            for cluster_column in match_result.group(4).strip().split(','):
                cluster_columns.append(cluster_column.replace('`', '').strip())
            return ClusterInfo(int(match_result.group(7).strip()), cluster_columns)

        return None

    def get_table_partition_columns(self, database_name, table_name):
        ddl_sql = self.get_ddl_sql(database_name, table_name)
        partition_columns = []
        ddl_format_sql = sqlparse.format(ddl_sql, reindent=True, keyword_case='upper')
        match_result = re.match(r'(.*?)PARTITION(.*?)BY(.*?)\((.*?)\).*', ddl_format_sql, re.S)
        if match_result:
            for partition_column in match_result.group(4).strip().split(','):
                partition_columns.append(partition_column.replace('`', '').strip())
            return partition_columns
        return None

    def get_primary_key(self, database_name, table_name):
        ddl_sql = self.get_ddl_sql(database_name, table_name)
        primary_keys = []
        ddl_format_sql = sqlparse.format(ddl_sql, reindent=True, keyword_case='upper')
        match_result = re.match(r'(.*?)UNIQUE(.*?)KEY(.*?)\((.*?)\).*', ddl_format_sql, re.S)
        if match_result:
            for primary_key in match_result.group(4).strip().split(','):
                primary_keys.append(primary_key.replace('`', '').strip())
            return primary_keys
        return None

    def get_table_columns(self, database_name, table_name) -> list[Column]:
        result = self.execute_sql(f"desc {database_name}.{table_name}")
        table_columns = []
        for row in result:
            table_columns.append(Column(name=row[0], type=row[1].upper(),
                                        is_null=True if row[2].strip() == 'Yes' else False,
                                        default_value=row[4]))
        return table_columns

    def execute_sql(self, sql, bind_params=None):
        try:
            self.connect()
            with self.connection.cursor() as cur:
                cur.execute(sql, bind_params)
                return cur.fetchall()

        except SourceExecutionError as e:
            logger.error(f"Doris connector execute sql {sql} failed, error: {e}")
            raise f"Doris connector execute sql {sql} failed, error: {e}"

    def type_mapping(self):
        return {
            'DATETIME': 'TIMESTAMP',
        }

    def close(self):
        if self.connection is not None:
            self.connection.close()
            self.connection = None
            logger.info(f"Close connection to Doris {self.connection_params['host']} successfully")

    def unload_data(self, task):
        logger.info(f"Begin to unload data from Doris table {task.name} to object storage")
        file_path = f"{task.project_id}/{task.id}/{task.name.split('.')[1]}/"
        object_storage_conf = object_storage_util.get_object_storage_config(self.storage_config_path)
        partitions = ''
        if hasattr(task, 'transform_partitions') and task.transform_partitions:
            partitions = ' PARTITION ( '
            values = task.transform_partitions[0]
            for index, value in enumerate(values):
                if index == 0:
                    partitions += value
                else:
                    partitions += f",{value}"
            partitions += ' ) '
        job_label = f"{task.name.split('.')[0]}_{task.name.split('.')[1]}_{datetime.now().strftime('%Y%m%d%H%M%S%f')}"
        unload_sql = f"EXPORT TABLE {task.name}\n" \
                     f"{partitions}\n " \
                     f"TO \"s3://{object_storage_conf['bucket']}/{file_path}\" \n" \
                     f"PROPERTIES(\n" \
                     f"\"label\" = \"{job_label}\" \n" \
                     f")\n" \
                     f"WITH S3\n" \
                     f"(\n" \
                     f"\"AWS_ENDPOINT\"=\"{object_storage_conf['endpoint']}\",\n" \
                     f"\"AWS_ACCESS_KEY\"=\"{object_storage_conf['id']}\",\n" \
                     f"\"AWS_SECRET_KEY\"=\"{object_storage_conf['key']}\",\n" \
                     f"\"AWS_REGION\"=\"{object_storage_conf['region']}\"\n" \
                     f");"
        logger.info(f"Unload data from Doris {self.connection_params['host']} with sql: {unload_sql}")
        try:
            self.execute_sql(unload_sql)
        except BaseException as e:
            logger.error(f"Unload data from Doris {self.connection_params['host']} failed, error: {e}")
            raise BaseException(f"Unload data from Doris {self.connection_params['host']} failed, error: {e}")
        check_unload_status_sql = f"SHOW EXPORT FROM {task.name.split('.')[0]} WHERE LABEL = '{job_label}';"
        while True:
            result = self.execute_sql(check_unload_status_sql)
            if len(result) > 0 and result[0][2] == 'FINISHED':
                break
            time.sleep(5)
        logger.info(f"Unload data from Doris {self.connection_params['host']} to path {file_path} successfully")
        return [file_path]

    def int_type_string(self):
        return 'INT'



from migration.scheduler.task.base_task import Task
from migration.scheduler.task.schema_task import SchemaMigrationTask
from migration.scheduler.task.data_task import DataMigrationTask

__all__ = ["Task", "SchemaMigrationTask", "DataMigrationTask"]


import logging

from migration.base import ProfileConfigError
from migration.connector.destination.base import Destination
from migration.connector.source import Source
from migration.scheduler.scheduler import SchedulerConfig, Scheduler
from migration.scheduler.transformer import Transformer
from migration.scheduler.data_transformer.transformer import DataTransformer
from migration.scheduler.data_validation.validation import Validation
from migration.scheduler.schema_transformer.transformer import SchemaTransformer

logger = logging.getLogger(__name__)

DOT_SPLITTER = '.'
LEFT_BRACKET = '('


class UnifyTransformer(Transformer):
    def __init__(self, source: Source, destination: Destination, project_name: str, db_list=None,
                 config_table_list=None,
                 external_table_list=None,
                 concurrency=1, quit_if_fail=False):
        super().__init__(source, destination, project_name, db_list, config_table_list, external_table_list,
                         concurrency, quit_if_fail)
        self.schema_tasks = []
        self.data_tasks = []
        self.validate_tasks = []
        self.schema_transformer = SchemaTransformer(source=self.source, destination=self.destination,
                                                    project_name=self.project_name, db_list=self.db_list,
                                                    config_table_list=self.config_table_list,
                                                    external_table_list=self.external_table_list,
                                                    concurrency=self.transform_concurrency,
                                                    quit_if_fail=self.quit_if_fail)
        self.data_transformer = DataTransformer(source=self.source, destination=self.destination,
                                                project_name=self.project_name, db_list=self.db_list,
                                                config_table_list=self.config_table_list,
                                                external_table_list=self.external_table_list,
                                                concurrency=self.transform_concurrency, quit_if_fail=self.quit_if_fail)
        self.validate_transformer = Validation(source=self.source, destination=self.destination,
                                               project_name=self.project_name, db_list=self.db_list,
                                               config_table_list=self.config_table_list,
                                               external_table_list=self.external_table_list,
                                               concurrency=self.transform_concurrency, quit_if_fail=self.quit_if_fail)

    def get_migration_tasks(self):
        self.schema_tasks = self.schema_transformer.get_migration_tasks()
        self.data_tasks = self.data_transformer.get_migration_tasks()
        self.validate_tasks = self.validate_transformer.get_migration_tasks()

    def schedule_migration_tasks(self):
        assert len(self.schema_tasks) == len(self.data_tasks) == len(self.validate_tasks)

        temp_schedule_map = {}
        temp_task_map = {}
        for i in range(len(self.transform_concurrency)):
            schedule_config = SchedulerConfig(quit_if_failed=self.quit_if_fail)
            scheduler = Scheduler(schedule_config)
            temp_schedule_map[i] = scheduler
            temp_task_map[i] = []
        for i in range(len(self.schema_tasks)):
            temp_task_map[i % len(self.transform_concurrency)].append(self.schema_tasks[i])
            temp_task_map[i % len(self.transform_concurrency)].append(self.data_tasks[i])
            temp_task_map[i % len(self.transform_concurrency)].append(self.validate_tasks[i])

        for i in range(len(self.transform_concurrency)):
            self.schedules_tasks_map[temp_schedule_map[i]] = temp_task_map[i]



import logging
import os
from datetime import datetime
from migration.connector.source.enum import Column
import json

logger = logging.getLogger(__name__)


class Source:
    def __init__(self, name, config: dict):
        self.name = name
        self.config = config
        self.source_id = self._gen_source_id()
        self.connection = None
        self.connection_params = None

    def get_connection_params(self):
        raise NotImplementedError

    def connect(self):
        raise NotImplementedError

    def get_database_names(self):
        raise NotImplementedError

    def get_table_names(self, database_name):
        raise NotImplementedError

    def get_table_columns(self, database_name, table_name) -> list[Column]:
        raise NotImplementedError

    def get_ddl_sql(self, database_name, table_name):
        raise NotImplementedError

    def get_table_cluster_info(self, database_name, table_name):
        raise NotImplementedError

    def get_table_partition_columns(self, database_name, table_name):
        raise NotImplementedError

    def get_primary_key(self, database_name, table_name):
        raise NotImplementedError

    def execute_sql(self, sql, bind_params=None):
        raise NotImplementedError

    def type_mapping(self):
        raise NotImplementedError

    def close(self):
        raise NotImplementedError

    def _gen_source_id(self):
        return f"{self.name}_{datetime.now().strftime('%Y%m%d%H%M%S%f')}"

    def __repr__(self):
        return f"<{self.__class__.__name__} {self.name}>"

    def unload_data(self, task):
        raise NotImplementedError

    def int_type_string(self):
        raise NotImplementedError

    def migration_test_connection(self):
        try:
            self.connect()
            self.close()
            return True
        except Exception as e:
            logger.error(f"Test connection to source failed: {e}")
            return False




import os
import platform
import subprocess

import setuptools

# Package metadata.

name = "clickzetta-migration"
description = "clickzetta migration tool"

# Should be one of:
# 'Development Status :: 3 - Alpha'
# 'Development Status :: 4 - Beta'
# 'Development Status :: 5 - Production/Stable'
release_status = "Development Status :: 3 - Alpha"
dependencies = [
    'schedule >= 1.1.0',
    'clickzetta-connector >= 0.8.34',
    'docopt >= 0.6.2',
    'pyyaml >= 6.0',
    'openpyxl >= 3.1.2',
    'tabulate >= 0.8.9',
    'pymysql >= 1.0.2',
    'sqlparse >= 0.4.1',
]
extras = {

}

all_extras = []

for extra in extras:
    all_extras.extend(extras[extra])

extras["all"] = all_extras

# Setup boilerplate below this line.

package_root = os.path.abspath(os.path.dirname(__file__))

version = {}
with open(os.path.join(package_root, "migration/version.py")) as fp:
    exec(fp.read(), version)
version = version["__version__"]

packages = setuptools.find_packages(exclude=["test", "test.*", "scripts", "scripts.*"])

setuptools.setup(
    name=name,
    version=version,
    description=description,
    url='https://www.yunqi.tech/',
    author="mocun",
    author_email="hanmiao.li@clickzetta.com",
    platforms="Posix; MacOS X;",
    packages=packages,
    install_requires=dependencies,
    extras_require=extras,
    python_requires=">=3.6",
    entry_points={
        'console_scripts': ['cz-migration=migration.migration:main'],
    },
    include_package_data=True,
    zip_safe=False,
)



import json
import os


def get_object_storage_config(storage_conf_path):
    with open(storage_conf_path, 'r') as f:
        storage_config = json.load(f)
    config = {}
    if storage_config['storage.type'] == 'oss':
        config['type'] = 'oss'
        config['bucket'] = storage_config['cz.fs.oss.bucket']
        config['id'] = storage_config['cz.fs.oss.access.id']
        config['key'] = storage_config['cz.fs.oss.access.key']
        config['endpoint'] = storage_config['cz.fs.oss.endpoint']
        config['region'] = storage_config['cz.fs.oss.sts.region'] if 'cz.fs.oss.sts.region' in storage_config else storage_config['cz.fs.oss.endpoint'].split('.')[0]
    elif storage_config['storage.type'] == 'cos':
        config['type'] = 'cos'
        config['bucket'] = storage_config['cz.fs.cos.bucket']
        config['id'] = storage_config['cz.fs.cos.access.key']
        config['key'] = storage_config['cz.fs.cos.secret.key']
        config['endpoint'] = storage_config['cz.fs.cos.endpoint']
        config['region'] = storage_config['cz.fs.cos.region']
    else:
        raise Exception(f"Unsupported storage type:{storage_config['storage.type']}")

    return config
